#!/bin/bash

set -eufo pipefail

BUILD_DIR=.

# clear toolchain
echo "BUILDER: remove old tool chain files ..."
rm -f "configure config.sub config.guess config.status"
rm -f "depcomp install-sh missing libtool ltmain.sh"
rm -f "aclocal.m4 config.h config.h.in"

# Remove all old Makefile.in
echo "BUILDER: remove old Makefile.in ..."
find . -name Makefile.in | xargs rm -f

# Rename configure.ac, otherwise autoscan will read it and report errors
echo "BUILDER: rename configure.ac to configure.ac.bak ..."
mv configure.ac configure.ac.bak

# Tool autoscan:
# Input : src folders
# Output: configure.scan (manualy edit it to get configure.ac)
echo "BUILDER: run autoscan ..."
autoscan 

# we can't directly use configure.scan. 
# we always copy our configure.ac to build directory instead.
# so we MUST update configure.ac manaully and keep it correct
echo "BUILDER: configure.scan is updated; compare it with configure.ac.bak to create configure.ac"
echo "BUILDER: remove configure.scan ..."
rm -f configure.scan

# rename configure.ac back
echo "BUILDER: rename configure.ac.bak back to configure.ac ..."
mv configure.ac.bak configure.ac

# Tool autoheader
# Input : configure.ac
# Output: config.h.in
echo "BUILDER: run autoheader ..."
autoheader --force 

# Tool aclocal
# Input : configure.ac
# Output: aclocal.m4
echo "BUILDER: run aclocal ..."
aclocal --force

# Tool autoconf
# Input : aclocal.m4, configure.ac
# Output: configure
echo "BUILDER: run autoconf ..."
autoconf --force 

# ltmain.sh is required by automake, but no tool can auto generate it. 
echo "BUILDER: installing ltmain.sh ..."

# TODO -- find a better way to get ltmain.sh
# UPDATE -- libtoolize can install ltmain.sh properly
#cp /usr/share/libtool/config/ltmain.sh .
libtoolize --force --copy

# Tool automake
# Input : configure.ac, Makefile.am
# Output: Makefile.in, and other missing tools
echo "BUILDER: run automake ..."
automake --add-missing --copy

# remove useless files
echo "BUILDER: clean useless files ..."
rm -rf autom4te.cache autoscan.log config.log config.h.in~
